<template>
    ${P.getPartCode(item,'PANEL').code}
</template>
<#assign import_block>
import { PanelDetailModel,PanelRawitemModel,PanelTabPanelModel,PanelTabPageModel,PanelFieldModel,PanelContainerModel,PanelControlModel,PanelUserControlModel,PanelButtonModel } from '@/model/panel-detail';
import ${srfclassname('${ctrl.codeName}')}Model from './${srffilepath2(ctrl.codeName)}-${ctrl.getControlType()?lower_case}-model';
import CodeListService from "@/codelist/codelist-service";
import UIService from '@/uiservice/ui-service';
</#assign>
<#ibizinclude>
../@MACRO/CONTROL/CONTROL_HEADER-BASE.vue.ftl
</#ibizinclude>
<#ibizinclude>
../../@MACRO/PANEL.ftl
</#ibizinclude>
    /**
     * 接口实现
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    getDatas(): any[] {
        if (!this.layoutData) {
            return [];
        }
        return [this.layoutData];
    }

    /**
     * 接口实现
     *
     * @returns {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    getData() {
        return this.layoutData;
    }

    /**
     * 父级部件引用
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public parentRef?: any;

    /**
     * 面板数据对象
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public inputData?: any;

    /**
     * 操作栏模型数据
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public actionModel?: any;

    /**
     * UI数据对象
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public data:any = {};

    /**
     * 数据模型对象
     *
     * @type {${srfclassname('${ctrl.codeName}')}Model}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public dataModel:${srfclassname('${ctrl.codeName}')}Model = new ${srfclassname('${ctrl.codeName}')}Model();

    /**
     * 代码表服务对象
     *
     * @type {CodeListService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public codeListService:CodeListService = new CodeListService();

    <#if appde??>
    /**
     * 界面UI服务对象
     *
     * @type {${srfclassname('${appde.getCodeName()}')}UIService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */  
    public appUIService:${srfclassname('${appde.getCodeName()}')}UIService = new ${srfclassname('${appde.getCodeName()}')}UIService();
    <#else>
    /**
     * 界面UI服务对象
     *
     * @type {UIService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */  
    public appUIService:UIService = new UIService();
    </#if>

    /**
     * 视图布局是否加载
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public isLayoutLoadding: boolean = false;

    /**
     * 视图布局数据
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public layoutData:any = {};

    /**
     * 视图布局面板模型对象
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public layoutModelDetails:any = {};

    /**
     * 视图布局顶级成员名称
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public rootLayoutDetailNames: string[] = <@compress  single_line=true>[
        <#if ctrl.getRootPSPanelItems?? && ctrl.getRootPSPanelItems()??>
        <#list ctrl.getRootPSPanelItems() as panelItem>
        '${panelItem.name}'<#if panelItem_has_next>,</#if>
        </#list>
        </#if>
        ];</@compress>

    /**
     *  初始化布局
     *
     *  @public
     *  @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public async initLayout() {
        if (this.rootLayoutDetailNames.length > 0) {
            for (let i = 0; i < this.rootLayoutDetailNames.length; i++) {
                const name = this.rootLayoutDetailNames[i];
                const rootItem = this.layoutItems[name];
                if (!rootItem) {
                    return;
                }
                await this.initLayoutItem(rootItem);
            }
        }
        return true;
    }

    /**
     *  初始化布局项
     *
     *  @public
     *  @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public async initLayoutItem(layoutModelItem: any, index: number = 0) {
        const { name } = layoutModelItem;
        const layoutModelDetail = Util.getLayoutItemInstance(layoutModelItem);
        if (!index) {
            await layoutModelDetail.load(this.context, this.viewparams);
            this.$set(this.layoutModelDetails, name, layoutModelDetail);
            this.$set(this.layoutData, name, layoutModelDetail.getData());
        } else {
            layoutModelDetail.setIndex(index);
            await layoutModelDetail.load(this.context, this.viewparams);
            <#noparse>this.$set(this.layoutModelDetails, `${name}_${index}`, layoutModelDetail);</#noparse>
            <#noparse>this.$set(this.layoutData, `${name}_${index}`, layoutModelDetail.getData());</#noparse>
        }
        if (layoutModelDetail && layoutModelDetail.details) {
            if (layoutModelDetail.dataRegionType === 'MULTIDATA') {
                const multiData = layoutModelDetail.getData();
                if (multiData && multiData.length > 0) {
                    for (let i = 0; i < multiData.length; i++) {
                        for (let j = 0; j < layoutModelDetail.details.length; j++) {
                            const key = layoutModelDetail.details[j];
                            if (this.layoutItems[key]) {
                                await this.initLayoutItem(this.layoutItems[key], i);
                            }
                        }
                    }
                }
            } else {
                for (let i = 0; i < layoutModelDetail.details.length; i++) {
                    const key = layoutModelDetail.details[i];
                    if (this.layoutItems[key]) {
                        await this.initLayoutItem(this.layoutItems[key],index);
                    }
                }
            }
        }
    }

    /**
     * 视图布局面板项模型对象
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public layoutItems:any = {
        <@getLayoutItemModel panelItems=ctrl.getAllPSPanelItems() type='ITEMLAYOUT' view />
    };

    /**
     * 监听数据对象
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Watch('inputData', { deep: true, immediate: true } )
    public onInputDataChange(newVal: any, oldVal: any){
        if(newVal){
            this.computedUIData(newVal);
        }
    }

    /**
     * 生命周期
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public mounted () {
        this.afterMounted();
    }

    /**
     * 执行mounted后的逻辑
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public afterMounted () {
        const _this: any = this;
        _this.initLayout().then((result: any) => {
            _this.isLayoutLoadding = false;
        });
    }

    /**
     * 计算UI展示数据
     * 
     * @param codelistArray 代码表模型数组
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public computedUIData(newVal:any){
        if((this.dataModel.getDataItems instanceof Function) && this.dataModel.getDataItems().length >0){
            this.dataModel.getDataItems().forEach((item:any) =>{
                this.data[item.prop] = newVal[item.prop];
            })
        }
    }

    /**
     * 打开编辑数据视图
     *
     * @type {any}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public opendata(args: any[],fullargs?:any[],params?: any, $event?: any, xData?: any){
        if (this.parentRef.opendata && this.parentRef.opendata instanceof Function) {
            this.parentRef.opendata(args,fullargs,params, $event, xData);
        }
    }

    /**
     * 打开新建数据视图
     *
     * @type {any}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public newdata(args: any[],fullargs?:any[], params?: any, $event?: any, xData?: any) {
        if (this.parentRef.newdata && this.parentRef.newdata instanceof Function) {
            this.parentRef.newdata(args,fullargs,params, $event, xData);
        }
    }
    
    /**
     * 删除
     *
     * @param {any[]} datas
     * @returns {Promise<any>}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public async remove(datas: any[]): Promise<any> {
        if (this.parentRef.remove && this.parentRef.remove instanceof Function) {
            return this.parentRef.remove(datas);
        }
    }

    /**
     * 刷新
     *
     * @param {*} [args={}]
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public refresh(args: any = {}) {
        if (this.parentRef.refresh && this.parentRef.refresh instanceof Function) {
            this.parentRef.refresh(args);
        }
    }

    /**
     * 处理值改变
     *
     * @public
     * @params args 改变数据
     * @params index 多数据容器下标
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public handleValueChange(args: { name: string, value: any }, index?: number) {
        if (!args || !args.name || Object.is(args.name, '') || !this.layoutData.hasOwnProperty(args.name)) {
            return;
        }
        const { name, value } = args;
        this.layoutData[name] = value;
        this.layoutModelDetails[name].setData(value);
        this.panelLogic(name, index);
        this.computeButtonAuthState();
        this.panelEditItemChange(name, value);
    }

    /**
     * 计算按钮权限状态
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public computeButtonAuthState() {
        Object.values(this.layoutModelDetails).forEach((layoutModel: any) => {
            if (layoutModel.itemType == 'BUTTON') {
                layoutModel.computeActionAuthState();
            }
        })
    }

    /**
     * 面板编辑项值变化后续操作
     *
     * @public
     * @param property 编辑项名
     * @param value 编辑项值
     * @returns {void}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public panelEditItemChange(property: string, value: any){
        // 面板数据变化事件
        if((this.dataModel.getDataItems instanceof Function) && this.dataModel.getDataItems().length >0){
            let modelitem =this.dataModel.getDataItems().find((item:any) =>{
                return item.name === property;
            }) 
            if(modelitem){
                this.$emit('panelDataChange',{[modelitem.prop]: value});
            }
        }
    }

    /**
     * 面板逻辑
     *
     * @public
     * @params name 改变项名称
     * @params index 多数据容器下标
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public panelLogic(name: string, index?: number): void {
        if (name && (index || index === 0)) {
            <#noparse>name = name.replace(`_${index}`,"");</#noparse>
        }
    <#list ctrl.getAllPSPanelItems() as panelItem>
        <@getPanelItemLogic panelItem />
    </#list>
    }

    <@handleButtonClick panelItems=ctrl.getAllPSPanelItems() item=ctrl />

    <#if ctrl.getPSUIActions?? && ctrl.getPSUIActions()??>
    <#list ctrl.getPSUIActions() as uiAction>
    <#if !P.exists("importService", uiAction.getFullCodeName(), "")>
    <#if !(uiAction.getPSAppDataEntity?? && uiAction.getPSAppDataEntity()??) || (uiAction.getPSAppDataEntity?? && uiAction.getPSAppDataEntity()?? && uiAction.getUIActionMode?? && (uiAction.getUIActionMode() == 'SYS' || uiAction.getUIActionMode() == 'CUSTOM'))>
    <@viewLayoutPanelUIAction item=uiAction />
    </#if>
    </#if>
    </#list>
    </#if>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>

